#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print tikz lines which are the rows of the SB tree.  These go clockwise
# around and show the region of p,q coverage.

use strict;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals;
use MyPrintwrap ':all';

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
my $x_limit = 200;
my $y_limit = 200;
my $is_outside = sub {
  my ($x,$y) = @_;
  return ($x > $x_limit || $y > $y_limit);
};
my $fmt = sub {
  my ($x) = @_;
  if ($x != int($x)) {
    return sprintf '%.2f', $x;
  } else {
    return $x;
  }
};
my $fmt_xy = sub {
  my ($x,$y) = @_;
  return '('.$fmt->($x).','.$fmt->($y).')';
};

print "  % generated by tools/$FindBin::Script\n";
my $scale = 0;
foreach my $depth (0 .. 5) {

  my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
  my ($prev_x,$prev_y) = $path->n_to_xy($n_lo);

  $Printwrap = 2;
  if ($n_lo == $n_hi) {
    printwrap("\\fill ($prev_x,$prev_y) circle(.04);\n");
    next;
  }

  printwrap("\\draw ($prev_x,$prev_y)");
  $Printwrap += 2;
  my $join = '--';

  foreach my $n ($n_lo+1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);

    my $x1 = $prev_x;
    my $y1 = $prev_y;
    my $x2 = $x;
    my $y2 = $y;
    if ($is_outside->($x1,$y1) && $is_outside->($x2,$y2)) {
      ### both outside ...

    } elsif ($is_outside->($x1,$y1)) {
      ($x1,$y1) = line_end_clip ($x2,$y2, $x1,$y1, $x_limit,$y_limit);
      printwrap(";\n");
      $Printwrap = 2;
      printwrap("\\draw ".$fmt_xy->($x1,$y1)."--".$fmt_xy->($x2,$y2));
      $Printwrap += 2;

    } elsif ($is_outside->($x2,$y2)) {
      ### exiting ...
      ($x2,$y2) = line_end_clip ($x1,$y1, $x2,$y2, $x_limit,$y_limit);
      printwrap ("--".$fmt_xy->($x2,$y2));

    } else {
      printwrap ("--".$fmt_xy->($x2,$y2));
    }
    # if ($scale) {
    #   $x = sprintf '%.1f', $x;
    #   $y = sprintf '%.1f', $y;
    # }
    $prev_x = $x;
    $prev_y = $y;
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
